#!/usr/bin/env python
import sys
import os
import numpy as np
import h5py
import re
from puq.jpickle import unpickle
import xml.etree.ElementTree as xml

sys.stdout = open("analyze.out", 'w')
sys.stderr = open("analyze.err", 'w')


# Restore the state of a PUQ session from a HDF5 file.
def load_from_hdf5(name):
    h5 = h5py.File(name, 'r+')
    sw = unpickle(h5['private/sweep'].value)
    sw.fname = os.path.splitext(name)[0]
    h5.close()

    sw.psweep._sweep = sw

    if hasattr(sw.psweep, 'reinit'):
        sw.psweep.reinit()
    return sw


# Plots probability curves
def plot_pdf_curve(xvals, vname, percent, desc):
    print 'plot_pdf_curve %s %s %s' % (vname, percent, desc)
    # compute upper and lower percentiles
    pm = (100 - percent)/200.0
    pp = 1 - pm

    # collect data into an array
    xarr = np.empty(len(xvals[vname]))
    yp = np.empty(len(xvals[vname]))
    ym = np.empty(len(xvals[vname]))
    for vindex in sorted(xvals[vname].keys()):
        xarr[vindex] = xvals[vname][vindex]
        yp[vindex] = pcurves[vname][vindex].ppf(pp)
        ym[vindex] = pcurves[vname][vindex].ppf(pm)

    curve = xml.SubElement(dout, 'curve', {'id': 'curve_pdf-%s-%s' % (vname, percent)})
    about = xml.SubElement(curve, 'about')
    if percent == 0:
        xml.SubElement(about, 'label').text = "mean"
    else:
        xml.SubElement(about, 'label').text = "middle %s%%" % percent
    xml.SubElement(about, 'group').text = '%s (Probability)' % desc
    comp = xml.SubElement(curve, 'component')
    xy = xml.SubElement(comp, 'xy')
    pts = ""
    for x, y in zip(xarr, yp):
        pts += "%s %s " % (x, y)
    if percent == 0:
        pts += '\n'
    else:
        for x, y in reversed(zip(xarr, ym)):
            pts += "%s %s " % (x, y)
        pts += "%s %s\n" % (xarr[0], yp[0])
    xy.text = pts


def plot_pdf(v, pdf, desc):
    id = '%s (PDF)' % desc
    elem = xml.SubElement(dout, 'curve', {'id': id})
    about = xml.SubElement(elem, 'about')
    xml.SubElement(about, 'label').text = id
    yaxis = xml.SubElement(elem, 'yaxis')
    xml.SubElement(yaxis, 'label').text = 'Probability'

    component = xml.SubElement(elem, 'component')
    xy = xml.SubElement(component, 'xy')
    pts = "%s 0\n" % pdf.x[0]
    for x, y in zip(pdf.x, pdf.y):
        pts += "%s %s\n" % (x, y)
    pts += "%s 0\n" % pdf.x[-1]
    xy.text = pts


sw = load_from_hdf5(sys.argv[1])
sw.analyze()

h5 = h5py.File(sys.argv[1], 'r+')
dtree = xml.parse('run_uq.xml')
droot = dtree.getroot()
dout = droot.find('output')

# curves built from pdfs
pcurves = {}
xvals = {}

regexp = re.compile('([ \da-z]+)\[([ \d]+)\]')

uqtype = h5.attrs['UQtype']
for v in h5[uqtype]:
    rs = unpickle(h5['/%s/%s/response' % (uqtype, v)].value)
    pdf = rs.pdf(fit=False)
    odata = h5['/output/data/%s' % v]

    # For curves built from pdfs, just put them in a dict for now
    if 'x' in odata.attrs:
        matches = regexp.findall(v)
        vname, vindex = matches[0]
        vindex = int(vindex)
        if vname not in pcurves:
            pcurves[vname] = {}
            xvals[vname] = {}
        xvals[vname][vindex] = odata.attrs['x']
        pcurves[vname][vindex] = pdf
    else:
        desc = h5['/output/data/%s' % v].attrs['description']
        plot_pdf(v, pdf, desc)

# now do pdf curves
for vname in xvals:
    desc = h5['/output/data/%s[0]' % vname].attrs['description']
    plot_pdf_curve(xvals, vname, 0, desc)
    plot_pdf_curve(xvals, vname, 50, desc)
    plot_pdf_curve(xvals, vname, 95, desc)

with open('run_uq.xml', 'w') as f:
    f.write("<?xml version=\"1.0\"?>\n")
    dtree.write(f)

with open('run_uq2.xml', 'w') as f:
    f.write("<?xml version=\"1.0\"?>\n")
    dtree.write(f)
h5.close()
